@@ -113,6 +113,7 @@ static void
 panfrost_batch_cleanup(struct panfrost_context *ctx,
                        struct panfrost_batch *batch)
 {
+   struct panfrost_screen *screen = pan_screen(ctx->base.screen);
    struct panfrost_device *dev = pan_device(ctx->base.screen);
 
    assert(batch->seqnum);
@@ -120,6 +121,8 @@ panfrost_batch_cleanup(struct panfrost_context *ctx,
    if (ctx->batch == batch)
       ctx->batch = NULL;
 
+   screen->vtbl.cleanup_batch(batch);
+
    unsigned batch_idx = panfrost_batch_idx(batch);
 
    pan_bo_access *flags = util_dynarray_begin(&batch->bos);
@@ -177,7 +180,7 @@ panfrost_get_batch(struct panfrost_context *ctx,
 
    /* The selected slot is used, we need to flush the batch */
    if (batch->seqnum) {
-      perf_debug_ctx(ctx, "Flushing batch due to seqnum overflow");
+      perf_debug(ctx, "Flushing batch due to seqnum overflow");
       panfrost_batch_submit(ctx, batch);
    }
 
@@ -227,7 +230,7 @@ panfrost_get_fresh_batch_for_fbo(struct panfrost_context *ctx,
     * draw/clear queued. Otherwise we may reuse the batch. */
 
    if (batch->draw_count + batch->compute_count > 0) {
-      perf_debug_ctx(ctx, "Flushing the current FBO due to: %s", reason);
+      perf_debug(ctx, "Flushing the current FBO due to: %s", reason);
       panfrost_batch_submit(ctx, batch);
       batch = panfrost_get_batch(ctx, &ctx->pipe_framebuffer);
    }
@@ -300,7 +303,7 @@ panfrost_batch_uses_resource(struct panfrost_batch *batch,
                              struct panfrost_resource *rsrc)
 {
    /* A resource is used iff its current BO is used */
-   uint32_t handle = panfrost_bo_handle(rsrc->image.data.bo);
+   uint32_t handle = panfrost_bo_handle(rsrc->bo);
    unsigned size = util_dynarray_num_elements(&batch->bos, pan_bo_access);
 
    /* If out of bounds, certainly not used */
@@ -364,11 +367,10 @@ panfrost_batch_read_rsrc(struct panfrost_batch *batch,
 {
    uint32_t access = PAN_BO_ACCESS_READ | panfrost_access_for_stage(stage);
 
-   panfrost_batch_add_bo_old(batch, rsrc->image.data.bo, access);
+   panfrost_batch_add_bo_old(batch, rsrc->bo, access);
 
    if (rsrc->separate_stencil)
-      panfrost_batch_add_bo_old(batch, rsrc->separate_stencil->image.data.bo,
-                                access);
+      panfrost_batch_add_bo_old(batch, rsrc->separate_stencil->bo, access);
 
    panfrost_batch_update_access(batch, rsrc, false);
 }
@@ -380,11 +382,10 @@ panfrost_batch_write_rsrc(struct panfrost_batch *batch,
 {
    uint32_t access = PAN_BO_ACCESS_WRITE | panfrost_access_for_stage(stage);
 
-   panfrost_batch_add_bo_old(batch, rsrc->image.data.bo, access);
+   panfrost_batch_add_bo_old(batch, rsrc->bo, access);
 
    if (rsrc->separate_stencil)
-      panfrost_batch_add_bo_old(batch, rsrc->separate_stencil->image.data.bo,
-                                access);
+      panfrost_batch_add_bo_old(batch, rsrc->separate_stencil->bo, access);
 
    panfrost_batch_update_access(batch, rsrc, true);
 }
@@ -451,11 +452,14 @@ panfrost_batch_to_fb_info(const struct panfrost_batch *batch,
                           struct pan_image_view *zs, struct pan_image_view *s,
                           bool reserve)
 {
+   struct panfrost_device *dev = pan_device(batch->ctx->base.screen);
+
    memset(fb, 0, sizeof(*fb));
    memset(rts, 0, sizeof(*rts) * 8);
    memset(zs, 0, sizeof(*zs));
    memset(s, 0, sizeof(*s));
 
+   fb->tile_buf_budget = dev->optimal_tib_size;
    fb->width = batch->key.width;
    fb->height = batch->key.height;
    fb->extent.minx = batch->minx;
@@ -463,6 +467,7 @@ panfrost_batch_to_fb_info(const struct panfrost_batch *batch,
    fb->extent.maxx = batch->maxx - 1;
    fb->extent.maxy = batch->maxy - 1;
    fb->nr_samples = util_framebuffer_get_num_samples(&batch->key);
+   fb->force_samples = pan_tristate_get(batch->line_smoothing) ? 16 : 0;
    fb->rt_count = batch->key.nr_cbufs;
    fb->sprite_coord_origin = pan_tristate_get(batch->sprite_coord_origin);
    fb->first_provoking_vertex = pan_tristate_get(batch->first_provoking_vertex);
@@ -691,7 +696,7 @@ void
 panfrost_flush_all_batches(struct panfrost_context *ctx, const char *reason)
 {
    if (reason)
-      perf_debug_ctx(ctx, "Flushing everything due to: %s", reason);
+      perf_debug(ctx, "Flushing everything due to: %s", reason);
 
    struct panfrost_batch *batch = panfrost_get_batch_for_fbo(ctx);
    panfrost_batch_submit(ctx, batch);
@@ -709,7 +714,7 @@ panfrost_flush_writer(struct panfrost_context *ctx,
    struct hash_entry *entry = _mesa_hash_table_search(ctx->writers, rsrc);
 
    if (entry) {
-      perf_debug_ctx(ctx, "Flushing writer due to: %s", reason);
+      perf_debug(ctx, "Flushing writer due to: %s", reason);
       panfrost_batch_submit(ctx, entry->data);
    }
 }
@@ -726,7 +731,7 @@ panfrost_flush_batches_accessing_rsrc(struct panfrost_context *ctx,
       if (!panfrost_batch_uses_resource(batch, rsrc))
          continue;
 
-      perf_debug_ctx(ctx, "Flushing user due to: %s", reason);
+      perf_debug(ctx, "Flushing user due to: %s", reason);
       panfrost_batch_submit(ctx, batch);
    }
 }
